#------------------------------------------------------------
#	Routines for the bookmarks editor.
#------------------------------------------------------------

 
package bookmarks;

use vars qw( $DEBUG );


# Main constructor.
# THe only arg needed is ( top level window ) passed to the GUI

sub new 
{
	my $class=shift;
	my $self={};

	$self->{file}="$ENV{HOME}/.elfftp-marks";
	$self->{DEBUG}=0;
	$self->{gui_args}=shift;

	$self->{bookmarks}=[];	# array of hashrefs.	

	bless $self;
}

# 	Basic accessor method

sub get
{
	my $self=shift;
	return $self->{bookmarks};
}

# 	Call GUI constructor

sub make_gui
{
	my $self=shift;

	$self->read;

	$self->{gui}=bookmarks::gui->new
	( 
	$self,
	$self->{bookmarks},
	$self->{gui_args}
	);
}
sub make_selector
{
	my $self=shift;

	$self->read;

	$self->{gui}=bookmarks::selector->new
	( 
	$self,
	$self->{bookmarks},
	$self->{gui_args}
	);
}

# 	Read bookmarks from disk

sub read
{
	my $self=shift;
	my $file=$self->{file};
	my @a;
	my @list;
	open ( R, "<$file" );
	while (<R>)
	{
		chomp;
		@a=split(/:/,$_);
		$#a==6 && push 
		( 
			@list, 
			{ 
				name	=>	$a[0],  
				address	=>	$a[1],
				port	=>	$a[2], 
				ldir	=>	$a[3], 
				rdir	=>	$a[4], 
				user	=>	$a[5], 
				pass	=>	$a[6]
			}	
		);
	} 
	close R;

	unless ( $#list > -1 && $list[0]->{name} eq "Template" )
	{

		unshift
		(
			@list, 
			{
				name 	=> 	"Template",
				address =>	"",
				port	=>	21,
				ldir	=>	$ENV{HOME},
				rdir	=> 	"/",
				user	=>	"",
				pass	=>	""
			}
		);
	}

	
	foreach my $x(@list)
	{
		foreach my $y(values(%$x))
		{
			$self->{DEBUG} && print STDERR $y . "*";
		}
		$self->{DEBUG} && print STDERR "\n";
	}
	$self->{bookmarks}=\@list;
} #read

# Write bookmarks to disk

sub write 
{
	my $self=shift;
	my $f=$self->{file};
	my $a_ref=$self->{bookmarks};

	open ( W, ">$f" );
	foreach my $item ( @$a_ref ) 
	{
	print W "$item->{name}:$item->{address}:$item->{port}:$item->{ldir}:$item->{rdir}:$item->{user}:$item->{pass}\n";
	}
	close W;
}

#------------------------------------------------------------
# 				GUI Routines.
#------------------------------------------------------------ 

package bookmarks::gui;
use vars qw(@ISA);
sub new 
{
	@ISA=qw(bookmarks);
	my $self=new bookmarks;
	my $class=shift;
	my $gui=shift;

	my $t=$self->{top}=$gui->get_top->Toplevel(-title=>"Bookmarks");

	my $row=2;	# for the grid manager
	$self->{list}=$t->Listbox->grid
	(
		-row=>1,
		-column=>1,
		-columnspan=>2,
		-sticky=>'ew'
	);
	$self->{scroll}=$t->Scrollbar->grid
	(
		-row=>1,
		-column=>4,
		-sticky=>'ns'
	);
	$self->{list}->configure( -yscrollcommand=>['set',$self->{scroll} ] );
	$self->{scroll}->configure( -command=>['yview',$self->{list}] );

	$self->{list}->bind 
	( 
		'<Button-1>',
		[
			sub 
			{ 
				my $list=shift; 
				my $self=shift; 
				$self->update_list($list,@_)   
			}, 
			$self,  $is_anon
		]
	);

	$self->{buttonframe}=$bf=$t->Frame();
	my $buttons=$self->{"buttons"}={};

	$buttons->{"add"}=$bf->Button
	(
		-text=>"clone",
		-command=> 
		[ 
			sub { my $self=shift; $self->add(@_) },
			$self
		]
	);

	$buttons->{"delete"}=$bf->Button
	(
		-text=>"delete",
		-command=> 
		[ 
			sub { my $self=shift; $self->delete(@_) } ,
			$self
		]
	);

	$buttons->{"new"}=$bf->Button
	(
		-text=>"new",
		-command=> 
		[ 
			sub { my $self=shift; $self->new_mark(@_) },
			$self
		]
	);

	$buttons->{"save"}=$bf->Button
	(
		-text=>"done",
		-command=> 
		[ 
			sub { my $self=shift;$self->write(@_) },
			$self,$file
		]
	);





	$buttons->{is_anon}=$bf->Checkbutton(
		-text=>"anonymous login",
		-variable=>$self->{is_anon},
		-command=>
		[ 
			sub { my $self=shift; $self->anon(@_) }, 
			$self, $is_anon 
		]
	);

	#---------------------------------------
	#	Layout and display widgets.
	#---------------------------------------

	$bf->grid(-row=>1,-column=>5);
	$buttons->{"add"} ->	grid(-row=>0,-column=>5);
	$buttons->{"delete"} ->	grid(-row=>1,-column=>5);
	$buttons->{"new"} ->	grid(-row=>2,-column=>5);
	$buttons->{"save"} ->	grid(-row=>3,-column=>5);
	$buttons->{"is_anon"}->grid(-row=>4,-column=>5);

	foreach my $x ( "name", "address", "port", "ldir", "rdir", "user", "pass")
	{
		$t->Label(-text=>$x)->grid(-row=>$row,-column=>1);
		$self->{$x}=$t->Entry(-width=>20)->grid(-row=>$row,-column=>2);
		$row++;
	}
	$self->{"pass"}->configure ( -show=>"*" );
	$self->{"port"}->configure ( -width=>5 );

	$self->read;
	&init_list ( $self, $self->{"list"} );
	$self->{readonly}=0;
	${$self->{is_anon}}=0;

	bless $self;

} #new

sub DESTROY
{
	$self->{top}->destroy;
}

#---------------------------------------------
#	This method is called by the constructor
#---------------------------------------------

sub init_list
{
	my $self=shift;
	my $list=shift;
	$self->{DEBUG} && print STDERR "init_list: $list\n\n";
#	my $self=shift;
	my $a_ref=$self->{bookmarks};

	$list->delete(0,'end');

	foreach my $item (@$a_ref)
	{
		$list->insert ( 'end', $item->{"name"} );
	}
} #init_list

#----------------------------------------------
# Accesor methods
#----------------------------------------------

sub get_top { return shift->{top}; } 


#----------------------------------------------
# 		(Private) Methods for the GUI
#----------------------------------------------


sub add
{
	my $self=shift;
	my $a_ref=$self->{bookmarks};
	my $a={};	# hashref
	my $item;
	foreach my $key ( "name","address","port","ldir","rdir","user","pass" )
	{
		$item=$self->{$key}->get()	;
		$a->{$key}=$item;
#		$self->{$key}->delete(0,'end');
	}
	$self->{"list"}->insert('end',$a->{name});
	push ( @$a_ref, $a );
}#add

sub new_mark
{
	my $self=shift;
	my $a_ref=$self->{bookmarks};
	my $a={};	# hashref
	
	foreach my $key (keys ( %{ $a_ref->[0] } ) )
	{
		$a->{$key} = $a_ref->[0]->{$key}
	}
	$a->{"name"}="New Bookmark";
	push ( @$a_ref, $a );
	$self->{"list"}->insert('end',"New Bookmark");
}

sub delete
{
	my $self=shift;
	my $a_ref=$self->{bookmarks};
	my $selected=($self->{list}->curselection)[0];

	if ( $selected == 0 ) { return 0; }

	$self->{"list"}->delete($selected);
	splice ( @$a_ref, $selected, 1 );
	foreach my $i(@$a_ref){
	}#foreach
	undef ($self->{selection});
}#delete

sub update_list
{
	my $self=shift;
	my $list=shift;

	my $a_ref=$self->{bookmarks};
	my $is_anon=$self->{is_anon};

	my $item;
	my $a={};

	my $selected=($list->curselection)[0];


	foreach my $key ( "name","address","port","ldir","rdir","user","pass" )
	{
		$item=$self->{$key}->get();	# save configuration
		$self->{$key}->configure(-state=>"normal");
		$a->{$key}=$item;
 		$self->{$key}->delete(0,'end');
		$self->{$key}->insert ( 'end', $a_ref->[$selected]->{$key}  );
	}	
	${$self->{is_anon}}=0;
	if ( defined ( $self->{selection} ) )
	{
		splice ( @$a_ref, $self->{selection}, 1, $a );
	}
	$self->{selection}=$selected;
	$self->init_list($list,$a_ref);
	$list->selectionSet($selected);

	if ( $self->{readonly} && $selected ) 
	{
		foreach my $key ( "name","address","port","ldir","rdir","user","pass" )
		{
		$self->{$key}->configure(-state=>"disabled");
		}
	}


} # update_list

sub anon
{
	my $self=shift;
	my $is_anon=$self->{is_anon};

	foreach my $key ( "user", "pass" )
	{
		$self->{$key}->configure(-state=>"normal");
		$self->{$key}->delete(0,'end');
	}
	if ( $$is_anon )
	{
		$self->{pass}->insert(0,"guest@");
		$self->{user}->insert(0,"anonymous");
		$self->{pass}->configure(-state=>"disabled");
		$self->{user}->configure(-state=>"disabled");

	}
} #anon

sub write 
{
	my $self=shift;
	my $file=shift;
	my $a_ref=$self->{bookmarks};
	my $a;
	my $item;

	if ( defined ( $self->{selection} ) )
	{
		splice ( @$a_ref, $self->{selection}, 1, $a );
	}

	foreach my $key ( "name","address","port","ldir","rdir","user","pass" )
	{
		$item=$self->{$key}->get();	# save configuration
		$a->{$key}=$item;
	}

	if ( defined ( $self->{selection} ) )
	{
		splice ( @$a_ref, $self->{selection}, 1, $a );
	}

	$self->{top}->destroy();
	
	$self->SUPER::write; # call to bookmarks::write

} # write

package bookmarks::selector;

@ISA=qw(bookmarks bookmarks::gui);

sub new
{

	shift;
	my $self=new bookmarks::gui(@_);
	$self->{config}={};

	#--------------------------------
	#	Destroy some unwanted buttons
	#--------------------------------


	foreach my $key ( keys( %{$self->{buttons}} ) )
	{
		$self->{buttons}->{$key}->destroy;
	}
	foreach my $key ( "name","user", "pass", "port", "ldir", "rdir", "address" )
	{
		$self->{$key}->configure ( -state=>"disabled");
	}


	#-------------------------------
	# Rewrite first bookmark.	

	$self->{bookmarks}->[0]->{name}="Connect to site";
	$self->{list}->delete(0);
	$self->{list}->insert(0,"Connect to site");

	#-------------------------------
	
	$self->{top}->configure ( -title=>"Connect" );
	my $bf=$self->{top}->Frame->grid(-row=>10,-column=>2);


	if ( $self->{DEBUG} ) 
	{
		foreach my $key ( "user","pass","port","ldir","rdir","address" )
		{ 
			print "self->$key: $self->{$key} \n"; 
		}
	}


	$self->{buttons}->{"connect"}=$bf->Button
	(
		-text=>"connect" ,
		-command=>
		[	
		sub 
		{ 
			my $self=shift;
			foreach my $key ( "user","pass","port","ldir","rdir","address" )
			{ 
				$self->{config}->{$key}=$self->{$key}->get(); 
				$self->{DEBUG} && print STDERR "self->config->$key : $self->{config}->{$key}\n";
			}
			$self->{top}->destroy;
		} ,
		$self
		]
		
	)
	-> grid(-row=>1,-column=>1);

	$self->{buttons}->{"cancel"}=$bf->Button
	(
		-text=>"cancel" ,
		-command=>
		[ 
		sub 
		{ 
			my $self =shift; 
			$self->{config}->{"connect"}=0; 
			$self->{top}->destroy;
		}, 

		$self 
		]
	)
	-> grid(-row=>1,-column=>2);

	$self->{readonly}=1;
	$self->{config}->{"connect"}=1;

	bless $self;	
}

#-----------------------------------
# accessor methods
#-----------------------------------

sub get_config { return shift->{config}; }



1;


__END__

=head1 NAME

bookmarks - Bookmark editor and editor widget

=head1 SYNOPSIS

    use bookmarks;
    
    $b = bookmarks->new("bookmarks_file", $gui);
	$b->make_gui;
	$b->read;
	$b->write;
	$list_ref=$b->get;
=head1 DESCRIPTION

C<bookmarks> is a class designed to edit bookmarks for a GUI ftp client.
It contains some simple methods ( a constructor, make_gui and accessor 
methods ) 

=head1 OVERVIEW

The bookmarks configuration file consists of lines with  the following format:

Bookmark-name:hostname:port:local_directory:remote_directory:username:password

These fields are further explained in the documentation of the get method.

=head1 CONSTRUCTOR

=over 4

=item new (bookmarks_file, gui)

This is the constructor for a bookmark object. bookmarks_file
is the bookmark config file, probably somewhere in the user's home directory
( eg a dotfile under $ENV{HOME} ). gui is a gui object as included in the 
ftp client. 
In fact any gui object will do as long as it has a toplevel method that
returns some Tk Toplevel widget.

=back

=head1 METHODS

=over 4

=item read

reads the contents of the bookmarks file the disk and loads it
into the bookmarks internal variable. THIS IS NOT AN ACCESSOR 
METHOD.

=item write

writes an array of anonymous hashes to the bookmarks file

=item get
Basic accessor method.
Returns a (anonymous) array of anonymous hashes , each hash has the following 
key value pairs:
 name, address, port, ldir, rdir,  user, pass. 
 These correspond to a name assigned to the bookmark, the host's 
 address, the port, the initial local directory, the initial remote
 directory, the user name, and password respectively.

=item make_gui

Starts the GUI (ie runs the read method then the GUI constructor).

=head1 COPYRIGHT

Copyright (c) 1999 Donovan Rebbechi
This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut
