package filemanager;

sub new
{
	my $class=shift;
	my $self={};

	# constants

	$self->{TMOUT}=12;
	$self->{BUFSIZE}=2000;
	$self->{MAX_LINK_DEPTH}=4;
	$self->{DEBUG}=1;

	# state variables

	$self->{connected}=0;
	$self->{rcwd}="/";			
	$self->{lcwd}="$ENV{HOME}/";

	# variable for local files in lcwd ( links/files/directories ) 
	#-------------------------------------------------------------
	my $lfile=$self->{lfile}={};
	$lfile->{f}=[];
	$lfile->{d}=[];
	$lfile->{l}=[];
	
	# variable for remote files in rcwd ( links/files/directories ) 
	#--------------------------------------------------------------
	my $rfile=$self->{rfile}={};
	$rfile->{f}=[];
	$rfile->{d}=[];
	$rfile->{l}=[];

	$self->{whichfile}=$self->{lfile}; 	# points to either lfile or rfile.
	$self->{selected}="";				# the "current" file
	$self->{ftp}=undef;	

	# state variable that tells us if the next operation is on 
	# local or remote files
	#---------------------------------------------------------
	$self->{local}=1;		

	# Preferences 
	$self->{pref}->{show_dotfiles}=1;

	bless $self;
}

sub setlocal 
{ 
	my $self=shift;
	$self->{local}=1; 
	$self->{whichfile}=$self->{lfile};	
} 
sub setremote 
{ 
	my $self=shift;
	$self->{local}=0; 
	$self->{whichfile}=$self->{rfile};
}

sub make_gui
{
	my $self=shift;
	my $gui=shift;
	$self->{gui}=$gui;
	$gui->setlocal;
	$self->ldir($gui);
}


sub connect 
{ 
	my $self=shift;
	my $gui=shift;
#	my $ftp_ref=shift;
	my $ftp=$self->{ftp};
	my $extra_args=shift;
	my @keys=keys(%$extra_args);
	my $conf={};

#	my ($hostname,$username,$password,$port,$connect)=$gui->connect ( $self->{connected},$gui,$extra_args );
	if ( $gui->isconnected == 0 ) 
	{ 
		my $selector=new bookmarks::selector($gui);
		$selector->get_top->waitWindow;
		$conf=$selector->get_config;
		$conf->{"connect"} || return;
	}



	my $msg="";
	my $fail=0; # 1 if connection attempt fails.


	if ( $gui->isconnected == 0 ) 
	{

		# Eval block is used to time out failed connection

#		eval 
#		{
#			alarm ( $self->{TMOUT} );
#			my $dns=`nslookup $conf->{address}`;
#			alarm ( 0 );
#		};	
#
#		if ( $@ ) 
#		{
#			if ( $@ =~ /timeout/ ) { 
#				$gui->message("DNS lookup failed. Maybe you aren't connected to the internet\n");
#				return 0; 
#				} else { 
#				die "unexpected exception" 
#			}
#		}
#
#
		eval 
		{
			alarm ($self->{TMOUT} );	
			$self->{ftp}=new Net::FTP
			(
				$conf->{address},
				Debug => $self->{DEBUG},
				Timeout => $self->{TMOUT}
			);
			alarm (0);
		}; #eval

		if ( $@ ) 
		{
			if ( $@ =~ /timeout/ ) 
			{ return; } 
			else
			{
				alarm(0);
				die "unexpected exception\n";
			}
		}

		if ( defined ( $self->{ftp} ) ) 
		{ 
			$self->{ftp}->login($conf->{user},$conf->{pass} );
			$msg=$self->{ftp}->code;

			if ( $msg =~ /^2/ )
			{
				$self->{ftp}->type("I");
				$self->{ftp}->cwd ("$conf->{rdir}");
				unless  ( $self->{ftp}->code =~ /^2/ ) 
				{ 
					$gui->message("couldn't CD to $conf->{rdir}"); 
				}
				$self->{connected}=1;

				$gui->setlocal;
				$self->setlocal;
				$self->{lcwd}=$conf->{ldir};
				$self->{lcwd}=~'/$' || $self->{lcwd}=~s/$/\// ;
				$self->ldir ( $gui );
				
				$gui->set_connect;
				$gui->setremote;
				$self->setremote;
				$self->dir ( $gui );
			}
			else { $fail=1; }

		} else { $fail=1; }

		if ( $fail ) 
		{
			$gui->message ("Couldn't connect to host $conf->{address}\n");
			# $gui->connect ( 1, $gui );
			$self->{connected}=0;
			undef ( $self->{ftp} ) ;
			&connect ( $self, $gui, ); 
		}
	

	}  # connected==0
	else 
	# $connected == 1 
	{ 
		$self->{ftp}->quit();
		$self->{DEBUG} && print STDERR "\nsending quit command to server ...\n";
		$self->{connected}=0;
		$gui->unset_connect;
	}
} # connect 

# launch bookmarks editor

sub bookmarks 
{
	my $self=shift;
	my $gui_args=shift;
	my $bookmarks=new bookmarks::gui( $gui_args);
}

sub dir 
{
#  	shorthands for globals
	my $self=shift;
	my $ftp=$self->{ftp};
	my $gui=shift;
	my $dirs 	= $self->{rfile}->{d}=[];	
	my $files 	= $self->{rfile}->{f}=[];
	my $links	= $self->{rfile}->{l}=[];

	# variables specific to subroutine
	my $buf;
	my $ls_out="";
	my @file_list=();
	my @dir_list=();	#output of dir
	my $ls_flags = ( $self->{prefs}->{show_dotfiles} ) ?
	"-a" : "";
	my $list_args=""; # args for LIST commands
	my $nlst_args=""; # args for NLST commands
	my $type="";
	my $systype="";

	$gui->setrcwd($self->{rcwd});

 	$self->{ftp}->quot('SYST');
	$systype = $self->{ftp}->message; 
	if ( $systype =~ /UNIX/ || $systype =~ /[Ww]indows/ ) { 
		$list_args="-la"; 
		$nlst_args="-a";
	}
	$self->{DEBUG} && print STDERR "SYSTEM TYPE : $systype";

	# set UNIX mode if the server runs Netware.
	if ( $systype =~ /[Nn]ovell/ ) { $self->{ftp}->quot('site nfs'); } 
	
	$type=$self->{ftp}->type("A"); #set type to ASCII for dir listing
	@dir_list=$self->{ftp}->dir($list_args);

	if ( $#dir_list >=0 && $dir_list[0] eq "." )
	{
		shift @dir_list;
	}

	if ( $self->{DEBUG} ) 
	{ 
		foreach (@dir_list) 
		{ 
			print STDERR "item: ";
			print STDERR ; 
			print STDERR "\n"; 
		}
	}


	unless ( $dir_list[0] eq ".."  )
	{ 
		unshift ( @dir_list , "drwxr-xr-x .." ); 
	}

	my $datacon=$self->{ftp}->nlst($nlst_args);	


	if ( defined ($datacon ) ) 
	{  # could we obtain a listing ? 
		while ( $datacon->read($buf,$self->{BUFSIZE},$self->{TMOUT}) ) 
		{ 
			$ls_out="$ls_out$buf";
		}
		$datacon->close();
		
	} #endif 

	$self->{ftp}->type("$type");	#restore transfer type.

	@file_list=split('\n',$ls_out );
	foreach my $item ( @file_list ) { chomp ( $item ); }
	if ( $file_list[0] eq "." )
	{
		shift @file_list;
	}
	unless ( $file_list[0] eq ".." || $systype =~ /VMS/ )
	{
		unshift (@file_list, ".." );
	}

	while ( $#dir_list > $#file_list ) 
	{
		shift ( @dir_list );	# get rid of "headers".
	}	
	if ( $#dir_list != $#file_list ) 
	{ 
		print STDERR "DIR_LIST\n\n";
		foreach my $item ( @dir_list ) { print STDERR "$item\n"; } 
		print STDERR "FILE_LIST\n\n";
		foreach my $item ( @file_list ) { print STDERR "$item\n"; } 
		print STDERR "\n\n*** corrupt directory listing *** \n\n" ; 

		exit 1;
	}

	unless ( $systype =~ "VMS" )	# VMS doesn't use perms field
	{

		foreach my $item ( @file_list ) 
		{ 
			if ( $dir_list[0] =~ /^d/ ) # check "permissions" field of dir output.
			{  
				push ( @$dirs, $item );
			} 
			elsif ( $dir_list[0] =~ /<DIR>\s*\Q$item\E/ ) # NT in "dos" mode
			{ 
				push ( @$dirs, $item );
			} 
			elsif ( $dir_list[0] =~ /^l/ ) 
			{ 
				push ( @$links, $item );
			} 
			else 
			{
				push ( @$files, $item );
			}
			shift ( @dir_list );
			
	
		}
	}

	if ( $systype =~ "VMS" ) 	# ugh! fix broken VMS dir listings
	{
		push ( @$dirs, ".." );

		foreach my $item ( @file_list ) 
		{
			if ( $item =~ /\.dir$/ ) 
			{
				$item =~ s/\.dir$//;
				push ( @$dirs, $item );
			}
			else 
			{
				push ( @$files, $item );
			}
		}
	}

	$gui->dir_output (  $self->{rfile}  );

}

sub ldir 
{
	# note : assume trailing slashes on directory $$cwd_ref
	my $self=shift;
	my $cwd=$self->{lcwd};
	my $fl=$self->{whichfile};
	my $gui=shift;

	$gui->setlcwd($self->{lcwd});

	$fl->{d}=[];
	$fl->{f}=[];
	$fl->{l}=[];
	$self->{DEBUG} && print STDERR "\n\n\nCWD is $cwd\n\n\n";
	my $ls_flags = ( $self->{prefs}->{show_dotfiles} ) ?
	"-a" : "";
	my $ls_out=`ls $ls_flags $cwd`;	# list of filenames
	my @list=split('\n',$ls_out);
	if ( $list[0] eq "." )
	{
	shift @list;
	}
	unless ( $list[0]=~/^\./ ) 
	{
	unshift ( @list, ".." );
	}
	foreach my $item ( @list ) 
	{ 
		chomp $item;
		if ( -d "$cwd$item" ) 
		{ 
			push ( @{$fl->{d}}, $item );
		} 
		elsif ( -l "$cwd$item" ) {
			push ( @{$fl->{l}}, $item );
		} 
		else 
		{
			push ( @{$fl->{f}}, $item );
		}
	} #foreach

	
	$gui->dir_output (  $fl  );
}

# "Quit" operation.
sub quit 
{
	my $self=shift;
	if ( defined ( $self->{ftp} ) && $self->{connected} ) 
	{ 
		$self->{ftp}->quit(); 
		print STDERR "closing FTP connection \n"; 
	} 
	print STDERR "CLOSE: Closing application\n";
	exit 0;
} #quit



sub cwd 
{
#	my $first_arg=shift;	# this is a widget if called from a bind, 0 if called from "get" button
	my $self=shift;
	$self->{DEBUG} && print STDERR "\n\nEnterring cwd : @_\n\n";
	my $ftp=$self->{ftp};
	my $lcwd=$self->{lcwd};
#	my $cwd=$self->{rcwd};
	my $rf=$self->{whichfile};
	my $gui=shift;
	
	my $dir_ref=$rf->{d};
	my $file_ref=$rf->{f};
	my $link_ref=$rf->{l};

	my $msg=0;
	my $d="";	# the directory we CD to
	my @cwd_indices=$gui->getlist;
	my $cwd_index=$cwd_indices[0];		# we only want the first one
	my $type="";
	my $rf_index=-1;

	$self->{DEBUG} && print STDERR "Enterring filetype ( $cwd_index ) from cwd\n";
	( $type, $rf_index ) = $self->filetype (  $cwd_index );
	$self->{DEBUG} && print STDERR "Exited filetype from cwd. returned $type , $rf_index \n";

	my $have_size = $self->{ftp}->supported("SIZE");
	

	if ( $type eq "d"  ) 
	{ 
#		if ( $first_arg == 0 ) { return; } 	# we don't download directories
		$d=${$rf->{d}}[$rf_index];
		$self->{DEBUG} && print STDERR "Changing to directory $d\n\n";
		$self->{ftp}->cwd( $d );
		$msg=$self->{ftp}->code();
		if ( $msg == 550 ) 
		{ 
			$gui->message ( "Couldn't CD. Sorry. Permission denied." ); 
			return -1; 
		}
		$self->{rcwd}=$self->{ftp}->pwd();
		$gui->setrcwd( $self->{rcwd} );

	}
	elsif ( $type eq "l" ) { 	# it's a link 
#		if ( $first_arg == 0 ) 
		{ 
		return; 
		} 	# we don't download links (yet)
		$d=${$rf->{l}}[$rf_index];
		$self->{ftp}->cwd( $d );
		$self->{rcwd}=$self->{ftp}->pwd();
		$gui->setrcwd( $self->{rcwd} );
		$msg=$self->{ftp}->code();
		if ( $msg == 550 ) 
		{ 	# cwd failed, so we'll try and download it, ie it probably points to a file.
		return $self->download($d,$gui);
		}
	}

	else 
	
	{	# it's a file. Let's download it !
	
	$d=${$rf->{f}}[$rf_index];
	$self->{selected}=$d;
	return $self->download($gui);

	} # big if : file/directory/link

	$self->dir ( $gui );
} 

sub download 
{
	my $self=shift;
	my $gui=shift;

	my $ftp=$self->{ftp};
	my $d=$self->{selected};
	my $lcwd=$self->{lcwd};
	my $bufsize=$self->{BUFSIZE};


	my $rsize=0;
	my $dataconn;
	my $buf="";
	my $msg="";
	my @fstat=();
	my $prompt="";
	my $size=0;
	my $progress;

	# Check to see if SIZE command is supported

	if ( $self->{ftp}->supported("SIZE")) 
	{
		$rsize=$self->{ftp}->size( $d );
		$msg=$self->{ftp}->code();
		if ( $msg == 550 ) 
		{ 
			$gui->message( "Unable to retreive file: server unwilling to complete request. It's probably a dangling symlink or something");
			return -1;
		}
	} 
	else 
	{
		$gui->message("This darned server doesn't support the \"size\" command, so I can't give progress info. Sorry.");
	}

	# check to see if downloaded file exists,
	# and prompt for reget/overwrite

	if ( -e "$lcwd$d" ) 
	{ 		
		@fstat=stat("$lcwd$d");
		if (  $fstat[7] >= $rsize  ) 
		{ 
			$prompt=$gui->download_prompt( 0 );
		} 
		else 
		{
			$prompt=$gui->download_prompt( 1 );
		}
	}

	if ( $prompt eq "Cancel" ) 
	{ 
		return; 
	} 
	elsif ( $prompt eq "Append" ) 
	{
		$self->{ftp}->rest($fstat[7]);
		$size=$fstat[7];
		open ( FILE, ">>$lcwd$d" ) || do 
		{ 
		$gui->message ( "cant open $lcwd$d for appending\n");
		};

	} 
	else 
	{
		open ( FILE, ">$lcwd$d" ) || do 
		{ 
		$gui->message ("cant open $lcwd$d for writing\n");
		};
	}

	$progress =  gui::progress_bar->new($gui) unless ( $rsize == 0 );
	
	#------------------
	# DOWNLOAD THE FILE
	#------------------
	
	$self->{DEBUG} && print STDERR "rsize is $rsize\n";
	$dataconn=$self->{ftp}->retr( $d );
	$msg=$self->{ftp}->code();
	if ( $msg == 550 ) 
	{
		$gui->message( "Unable to retreive file: server unwilling to complete request. It's probably a dangling symlink or something");
		return -1;
	}
	while ( $dataconn->read ( $buf, $bufsize, 10) )
	{
		print FILE $buf;
		if ( $rsize ) 
		{ 
			my $percent=int ( 100* $size/$rsize );
			$progress->update_bar( $percent, $size, $rsize);
#			$msg=$self->{ftp}->quot("NOOP");
#			$msg=~/^2/ || $gui->message( "server has stopped responding" );
		} 
		$size=$size+length($buf);
	} # end read loop

	#----------------------
	# Clean up time.
	#----------------------

	$dataconn->close;
	close FILE;
	$self->{DEBUG} && print STDERR "Closing data connection\n";
	$msg=$self->{ftp}->code();
	if ( $msg =~ /^2/ ) 
	{ 
		$self->{DEBUG} && print STDERR "it's done now\n"; 
		$gui->message ("File transfer complete!");
	} 
	else
	{
	$gui->message( "transfer incomplete, errno $msg\n" );
	}
	
	$self->setlocal;
	$gui->setlocal;
	$self->ldir($gui);
	$self->setremote;
	$gui->setremote;
	$msg=$self->{ftp}->quot("NOOP");
	$msg == 2 || $gui->message ( "server has stopped responding" );

	if ( $msg == 226 ) 
	{ 
		return 1; 
	} 
	else 
	{ 
		return 0; 
	} 
}




sub upload 
{
	my $self=shift;
	my $ftp=$self->{ftp};
	my $d=$self->{selected};
	my $lcwd=$self->{lcwd};
	my $gui=shift;		# gui hash

	my @fstat=stat("$lcwd$d");			# new: stat of file to be transferred
	my $rsize=$fstat[7];					# changed
	my $buf="";
	my $bufsize=$self->{BUFSIZE};
	my $msg="";
	my $dataconn=$self->{ftp}->stor( $d );			# changed

	$msg=$self->{ftp}->code();
	if ( $msg >= 500 ) { $gui->message ("upload unsuccessful",$gui); return ; } 

	my $progress =  new gui::progress_bar($gui);

	$self->{DEBUG} && print STDERR "rsize is $rsize\n";
	open ( FILE, "<$lcwd$d" ) || do 
		{
		$dataconn->close(); 
		$gui->message (  "cant open $lcwd$d for reading\n" );
		};
	my $size=0;
	my $bytesin=0;	# amount returned by sysread

	while ( sysread(FILE,$buf,$bufsize) )
	{
		$dataconn->write ( $buf, length($buf) , 10);
		my $percent=int ( 100* $size/$rsize );
		$progress->update_bar($percent, $size, $rsize);
		$size=$size+length($buf);
	} # end read loop

	$dataconn->close;
	close FILE;
	$msg=$self->{ftp}->code();
	if ( $msg == 226 ) {  $gui->message ( "Transfer complete!" ) } 
	$progress->DESTROY;
}

sub lcwd 
{

	my $self=shift;
	$self->{DEBUG} && print STDERR "\n\nEnterring lcwd : @_\n\n";


	my $fl=$self->{lfile};
	my $gui=shift;


	my $d="";	# the directory we CD to
	my @cwd_indices=$gui->getlist;
	my $cwd_index=$cwd_indices[0];		# we only want the first one
	my $type="";
	my $fl_index=-1;

	( $type, $fl_index ) = $self->filetype (  $cwd_index );
	if ( $type eq "d" ) { # it's a directory
#		if ( $first_arg == 0 ) { return; } 	# we don't upload directories
		$d=${$fl->{d}}[$fl_index];
		if ( -r "$self->{lcwd}$d" && -x "$self->{lcwd}$d" && $d ne "." ){
			$self->{DEBUG} && print STDERR "Changing to directory $d\n\n";
			if ( $d eq ".." ) 
			{
			$self->{lcwd}=~s#(.*/)[^/]*/#$1#;
			$gui->setlcwd($self->{lcwd});
			} 
			else
			{
			$self->{lcwd}="$self->{lcwd}$d/";
			$gui->setlcwd($self->{lcwd});
			}
		} else {
		$gui->message ( "permission denied\n" ) unless ( $d eq "." );
		}
	  }
	elsif ( $type eq "l" ) { 	# it's a link 
#		if ( $first_arg == 0 ) { return; } 	# we don't upload links either
		$d=${$fl->{l}}[$fl_index];
		$self->{selected}=$d;
		if ( -d ( readlink ( "$self->{lcwd}$d" ) ) ) 
		{
			$self->{lcwd}="$self->{lcwd}$d/";
		} 
		else 
		{
			$self->{DEBUG} && print STDERR "sorry, no can do. Link target" , readlink ("$self->{lcwd}$d"), "is not a directory\n";
		}
	}
	else {	# it's a file.
	$d=${$fl->{f}}[$fl_index];
	$self->{selected}=$d;
	if ( defined ( $self->{ftp} )) {
		$self->upload($gui);
		} else {
		$gui->message("Cannot upload file: not connected\n");
	}
	return;


	}
	$self->ldir (  $gui );
}

sub delete_file 
{
	my $self=shift;
	my $local=shift;
	my $gui=shift;

	my $ftp=$self->{ftp};
	my $lcwd=$self->{lcwd};
	my $f=$self->{whichfile};		# hash of arrayrefs of {dir,link,file}s

	my $d="";	# the thing we'll delete
	my @cwd_indices=$gui->getlist;
	my $cwd_index=$cwd_indices[0];		# we only want the first one
	
	my $type="";
	my $f_index=-1;

	( $type, $f_index ) = $self->filetype ( $cwd_index );
	
	$d=${$f->{$type}}[$f_index];

	if ( $type eq "f" || $type eq "l" ) 
	{ 
		if ( $local ) 
		{  		
			unlink ( "$lcwd$d" );  
		} 
		else 
		{
			$self->{ftp}->delete($d);
		}
	} 
	else 
	{ 
		if ( $gui->confirm("erase $d and all subdirectories ?") && $local )
		{
		system ("rm -rf $lcwd$d" ); 
		}
	}

	if ( $local ) 
	{ 
		$self->ldir ( $gui ) ;
	} 
	else 
	{
		$self->dir ( $gui  ) ;
	}
} 

sub l_make_dir 
{
	my $self=shift;
	my $cwd=$self->{lcwd};
	my $d=shift;
	my $mode=shift||755;
	my $gui=shift;

	unless ( $mode =~ /[0-7]{3,4}/ )
	{ 
		$gui->message ( "Invalid mode" ); 
		return 0; 
	}

	mkdir ( "$cwd/$d", oct($mode) );
	$self->ldir($gui);
	return 0;
}

sub r_make_dir
{
	my $self=shift;
	my $ftp=$self->{ftp};
	my $d=$self->{selected};
	my $mode=shift;
	my $gui=shift;
	my $msg;

	unless ( $mode eq "" ) 
	{
		unless ( $mode =~ /[0-7]{3,4}/ ) 
		{ 
			$gui->message ( "Invalid mode" ); 
			return 0; 
		}
	}

	$self->{ftp}->mkdir($d);
	$msg=$self->{ftp}->code;

	if ( $msg =~ /^2/ ) 
	{ 
		if ( defined ($mode)  )
		{
			$self->{ftp}->quot("SITE CHMOD $mode $d");
		}

		$msg=$self->{ftp}->code;

		unless ( $msg=~/^2/ ) 
		{ 
			$gui->message ("chmod failed");
		}

		return 1; 
	}	

	if ( $msg == 550 ) 
	{ 
		$gui->message("Couldn't create directory $d : Permission denied"); 
		return 0;
	}

	if ( $msg == 521 )
	{
		$gui->message("Couldn't create directory $d : File exists");
		return 0;
	}

	$gui->message("Couldn't create directory $d. Command failed");
	return 0;
}

sub r_rename 
{
	my $self=shift;
	my $to=shift;
	my $gui=shift;
	my $index=shift;
	my $msg;

	my $f=$self->{rfile};

	my $type="";
	my $f_index=-1;

	( $type, $f_index ) = $self->filetype ( $index );
	my $from=${$f->{$type}}[$f_index];

	$self->{ftp}->rename($from,$to);
	$msg=$self->{ftp}->code;
	if ( $msg =~ /^2/ ) 
	{ 
	$self->dir($gui);	
	return 1; 
	}
	$gui->message("Rename failed");
	return 0;
}

sub l_rename 
{
	my $self=shift;
	my $to=shift;
	
	my $gui=shift;
	my $index=shift;

	my $f=$self->{lfile};

	my $type="";
	my $f_index=-1;
	my $from;
	
	

	( $type, $f_index ) = $self->filetype ( $index );
	$from=${$f->{$type}}[$f_index];

	
	

	if ( $to =~ "^/" ) 
	{
		rename ( "$self->{lcwd}$from", "$to" );
	}
	else
	{
		rename ( "$self->{lcwd}$from", "$self->{lcwd}$to" );
	}

	$self->ldir($gui);

}

# filetype: returns type, and index of $fl->{"$type"} that goes with the item in question
sub filetype 
{
	my $self=shift;
	my $fl=$self->{whichfile};
	my @a=keys(%$fl); 
	my $cwd_index=shift;
	if ( $cwd_index <= $#{$fl->{d}} ) { 
		return ("d",$cwd_index) ; 
		} elsif ( $cwd_index <= ( ( $#{$fl->{d}} + 1 ) + $#{$fl->{l}} ) ) { 	# it's a link 
		return ("l",  $cwd_index-($#{$fl->{d}}+1) );	# subtract number of items in @$dir_ref
		} else {
		return ( "f", $cwd_index=$cwd_index-($#{$fl->{d}}+1)-($#{$fl->{l}}+1) ) ;
	}
}


#---------------------------------------------------------------# 
# 	getstats: 													# 
# 	recover stats from long output of unix dir list				#	
#---------------------------------------------------------------#


sub getstats 
{
	my $self=shift;
	my $string=shift;
	my $name=shift||"";		# this is more reliable if name is defined
	my $stats=
	{ 
		name=>"", 
		mode=>"", 	# this is a string such as drwxr-xr-x
		owner=>"", 
		group=>"", 
		hardlinks=>"", 
		type=>"", 	# first field of perms. "-" corresponds to a real file.
		size=>"", 
		date=>"" 
	};

	#	Horrendous regular expressions required to match dates and permission fields
	my $regex=
	{
		mode=>"[a-z-]([r-][w-][x-]){3}",
		date=>"[JFMASOND][a-z][a-z] *\\d\\d? *\\d+.\\d+"
	};


	chomp $string;
	chomp $name;

	$stats->{type}=substr ( $string, 0, 1 );

	#	Determine file name. We need to seperately address the case
	#	where we have a symbolic link

	if ( $stats->{type} eq "l" ) 	
	#	if it's a symbolic link
	{
		# the following will break if some idiot puts trailing spaces 
		# *or* '>' in the file name
		if ( $name eq "" )
		{
			$stats->{name}=$string;
			$stats->{name}=~s/\s*->\s*[^>]*$//;
			$stats->{name}=~s/.*$regex->{date} //;
		}
		else 
		{
			$stats->{name}=$name;	
		}
		$string=~s/\s*->[^>]*$//;

	}

	else	
	# if it's not a symbolic link
	{
		if (  $name eq ""   )
		{
			$stats->{name}=$string;
			$stats->{name}=~s/.*$regex->{date} //;
		}
		else
		{
			$stats->{name}=$name;
		}
	}	

	$string=~s/\s*\Q$stats->{name}\E$//;	# remove name from the string

	$stats->{date}=$string;
	my $d=$regex->{date};

	$stats->{date}=~s/.*($regex->{date})/$1/;

	# now remove date  from string
	$string=~s/\s*$regex->{date}\s*$//;

	$stats->{size}=$string;
	$stats->{size}=~s/.*\s+//;
	$stats->{size}=~/\d*/  || die "error: size is $stats->{size}\n";

	$string=~s/\d*$//;
	
	( $stats->{mode},$stats->{hardlinks}, $stats->{owner}, $stats->{group} ) = split /\s\s*/,$string;

	return $stats;
}

#-------------------------------------------------
# 	Some accessor methods

sub getlcwd { return shift->{lcwd}; }
sub getrcwd { return shift->{rcwd}; }


"Tom told me to put this here";
