#!/usr/bin/perl -w

package Tk::tlist;

use strict;
use Tk qw(Ev);

require Tk::Toplevel;
require Tk::Frame;
use vars qw($VERSION @ISA);
$VERSION = '0.1';

use base qw(Tk::Widget);
@ISA= qw(Tk::Derived Tk::Frame);

Tk::Widget->Construct('tlist');

sub Populate {
	my ( $self, $args) = @_;
	
	$self->SUPER::Populate($args);
	my $columns=delete $args->{"-columns"}||0;

	$self->{listbox}=[];
	#$self->{frame}=$toplevel->Frame();
	my @listbox=@{$self->{listbox}};
	unless ( $columns =~/^\d+$/ ) { return 0;  } # bad argument

	for ( my $count=0; $count <$columns; $count++ ){
	${$self->{listbox}}[$count]=$self->Listbox(-exportselection=>0)->pack(-side=>'left');
	}

	$self->{scroll}=$self->Scrollbar->pack(-fill=>'y',-expand=>'1');


	$self->{scroll}->configure ( 
		-command=> [
			\&multiscroll,
			$self->{scroll},
			$self->{listbox}
		]
	);

	$self->bind ( "Tk::Listbox","<B1-Leave>", "" );	#erase default bindings
#	$self->bind ( "Tk::Listbox","<B1-Leave>"=>[\&AutoScan,Ev('x'),Ev('y')]);

	foreach my $lb ( @{$self->{listbox}} ) {
		$lb->configure ( -yscrollcommand=>['set', $self->{scroll} ] );
		$lb->bind ( 
			'<ButtonPress-1>', 
				[ 
					sub { $self->update_selection(@_) },
					$lb 
				] 
			);
		$lb->bind ( 
			'<Motion>', 
				[ 
					sub { $self->update_selection(@_) },
					$lb 
				] 
			);
#		$lb->bind ( "<B1-Leave>"=>[\&AutoScan,$lb,Ev('x'),Ev('y')]);

	} #foreach	
		

} #Populate

sub multiscroll {  # multiscroll
	my ($sb,$wigs,@args) = @_;
	my $w;
	foreach $w (@$wigs) {
		$w->yview(@args);
	}
} #multiscroll

sub update_selection {
	my $self=shift;
	my $lb=shift;
	my @selection=$lb->curselection;
	foreach my $box( @{$self->{listbox}} ) {
		$box->selectionClear ( '0','end' );
		foreach my $item (@selection){
			$box->selectionSet($item);
		}
	}
}

sub insert {
	my $self=shift;
	my $first_index=shift;
	my @elts=@_;	# each item to be inserted should be an 
					# anonymous array with dimension same as o of columns.
	foreach my $item ( @elts ) {
		foreach my $lb ( @{$self->{listbox}} ) {
			$lb->insert($first_index,shift (@$item) );
		}
	$first_index++ unless ( $first_index eq 'end' );
	}
} #insert

sub delete {
	my $self=shift;
	my $first_index=shift;
	my $last_index;

	if ( $#_ > -1 ) { 
		$last_index=shift; 
		} else {
		$last_index=$first_index;
	}

	for ( my $i=$first_index; $i <= $last_index; $i++ )  {
		foreach my $lb ( @{$self->{listbox}} ) {
			$lb->delete($i );
		}
	}
} #delete

sub curselection {
	my $self=shift;
	return $self->{listbox}->[0]->curselection();
	}



# unused code ... 

sub AutoScan {
	my $listbox=shift;
	my $x=shift;
	my $y=shift;
	my $self=$listbox->parent;
	print STDERR "calling AutoScan on $self $x $y\n";

	foreach my $lb ( @{$self->{listbox}} ) { 
	$lb->AutoScan($x,$y) unless ( $lb == $listbox );
	}
}

__END__

=head1 NAME

Tk::tlist - multi column Tk List for perl.


=head1 SYNOPSIS

use Tk::tlist
my $l=new Tk::tlist ( -columns=> 3 );
$l->insert(3,["a","b","c"]);
$l->delete(4,'end');
my @curselection=$l->curselection;

=head1 DESCRIPTION

Creates a multicolumn listbox into which one can insert multicolumn
data. To be precise, a tlist consists of several listboxes, with a
scrollbar that simoultaneously scrolls all listboxes, and the property that
selecting the Nth row of one listbox selects the Nth row of the others.

=head1 METHODS

=item $listbox->insert($row,$arrayref);

This behaves similarly to the the insert method of Tk::Listbox, but 
instead of taking a scalar as the second argument, it takes a reference
to an array with one element corresonding to each column. For example,
to add three columns "a" "b" and "c" to a three column listbox, use
$listbox->insert('end',[ "a", "b", "c" ]);

=item $listbox->delete($lower [ , $upper] );

This works the same way as the delete method for a regular listbox

=item $listbox->curselection

Returns indices corresponding to selected items ( same as the curselection
method for a regular listbox )

